#include "DXUT.h"
#include "LightMaterial.h"

Light::Light(D3DLIGHTTYPE type)
{
	Clear();

	Type = type;	

	if( Type == D3DLIGHT_POINT )
	{
		//D3DXCreateSphere( pDevice, 2.0f, 10, 10, &m_pMesh, NULL );		
		SetDiffuseColor( Color::White );			

		// Point lights have no direction but do have a position
		Position =  D3DXVECTOR3(0.0f, 10.0f, 0.0f);

		Attenuation0 = 0.1f;    // no constant inverse attenuation
		Attenuation1 = 0.0125f;    // only .125 inverse attenuation
		Attenuation2 = 0.0f;    // no square inverse attenuation		
		Range = 100.0f;
	}
	else if( Type == D3DLIGHT_SPOT )
	{		
		SetDiffuseColor( Color::White );		

		// Spot lights have direction and a position
		// This one is place on top and beams a light downwards
		Position = D3DXVECTOR3(0.0f, 100.0f, 0.0f);
		Direction = D3DXVECTOR3(0.0f, -1.0f, 0.0f);
		Normalize();

		Range = 1000.0f;
		Theta = 0.5f;
		Phi = 0.5f;
		Falloff = 1.0f;
		Attenuation0 = 1.0f;
	}
	else
	{		
		SetDiffuseColor( Color::White );		
		Range      = 1000.0f;
		Direction = D3DXVECTOR3(1.0f, -1.0f, 1.0f);
		Normalize();		
	}

	SetSpecularColor(150, 150, 150);	
}

void Light::Clear() 
{
	SetDiffuseColor( 0.f, 0.f, 0.f );
	SetAmbientColor( 0.f, 0.f, 0.f );
	SetSpecularColor( 0.f, 0.f, 0.f );
	SetDirection( 0.f, 0.f, 0.f );
	SetRange( 0.f );
	SetFalloff( 0.f );
	SetAttenuation0( 0.f );
	SetAttenuation1( 0.f );
	SetAttenuation2( 0.f );
	SetTheta( 0.f );
	SetPhi( 0.f );
}

void Light::SetDiffuseColor(float r, float g, float b)
{
	Diffuse.r = RGB_MULTIPLIER * r;
	Diffuse.g = RGB_MULTIPLIER * g;
	Diffuse.b = RGB_MULTIPLIER * b;
}

void Light::SetSpecularColor(  float r, float g, float b )
{
	Specular.r = RGB_MULTIPLIER * r;
	Specular.g = RGB_MULTIPLIER * g;
	Specular.b = RGB_MULTIPLIER * b;	
}

void Light::SetAmbientColor(  float r, float g, float b )
{
	Ambient.r = RGB_MULTIPLIER * r;
	Ambient.g = RGB_MULTIPLIER * g;
	Ambient.b = RGB_MULTIPLIER * b;
}

void Light::SetPosition( float x, float y, float z )
{
	Position = D3DXVECTOR3( x, y, z );
}

void Light::SetDirection( float x, float y, float z )
{
	Direction = D3DXVECTOR3( x, y, z );
}

Material::Material()
{
	Clear();
	SetDiffuseColor( 255, 255, 255 );
	SetAmbientColor( 255, 255, 255 );
}

void Material::Clear() 
{
	SetDiffuseColor( 0.f, 0.f, 0.f );
	SetAmbientColor( 0.f, 0.f, 0.f );
	SetSpecularColor( 0.f, 0.f, 0.f );
	SetEmissiveColor( 0.f, 0.f, 0.f );
	SetPower( 0.f );
}

void Material::SetDiffuseColor(float r, float g, float b)
{
	Diffuse.r = RGB_MULTIPLIER * r;
	Diffuse.g = RGB_MULTIPLIER * g;
	Diffuse.b = RGB_MULTIPLIER * b;		
}

void Material::SetSpecularColor(  float r, float g, float b )
{
	Specular.r = RGB_MULTIPLIER * r;
	Specular.g = RGB_MULTIPLIER * g;
	Specular.b = RGB_MULTIPLIER * b;
}

void Material::SetAmbientColor(  float r, float g, float b )
{
	Ambient.r = RGB_MULTIPLIER * r;
	Ambient.g = RGB_MULTIPLIER * g;
	Ambient.b = RGB_MULTIPLIER * b;	
}

void Material::SetEmissiveColor(  float r, float g, float b )
{
	Emissive.r = RGB_MULTIPLIER * r;
	Emissive.g = RGB_MULTIPLIER * g;
	Emissive.b = RGB_MULTIPLIER * b;	
}