uniform extern float4x4 matWorld;
uniform extern float4x4 matVP;
uniform extern float3 dirToLight;
uniform extern float3 eyePos;

uniform extern texture t1;
uniform extern texture t2;
uniform extern texture t3;
uniform extern texture t4;
uniform extern texture t5;
uniform extern texture t6;
uniform extern texture t7;
uniform extern texture t8;
uniform extern texture t9;
uniform extern texture a;
uniform extern texture a2;
uniform extern texture a3;
uniform extern texture l;

struct VS_INPUT
{
   float4 position : POSITION0;   
   float3 normal : NORMAL0;
   float2 uv : TEXCOORD0;         // alpha UV
   float2 uv2 : TEXCOORD1;        // diffuse UV
};

struct VS_OUTPUT
{
   float4 position : POSITION0;
   float2 uv : TEXCOORD0;
   float2 uv2 : TEXCOORD1;
   float  shade : TEXCOORD2;   
};

VS_OUTPUT MainVS(VS_INPUT input)
{
   VS_OUTPUT output = (VS_OUTPUT)0;
   
   // Transform World, View and Projection
   float4 temp = mul(input.position, matWorld);
   output.position = mul(temp, matVP);

   // Directional Lighting
   output.shade = max(0.0f, dot(normalize(input.normal), dirToLight));
   output.shade = 0.2f + output.shade * 0.8f;

   // Copy UV coordinates
   output.uv = input.uv;
   output.uv2 = input.uv2;
   
   return output;
}

sampler alpha = sampler_state
{
	Texture = <a>;
	MinFilter = LINEAR;
	MagFilter = LINEAR;
	MipFilter = LINEAR;
};

sampler alpha2 = sampler_state
{
	Texture = <a2>;
	MinFilter = LINEAR;
	MagFilter = LINEAR;
	MipFilter = POINT;
	AddressU  = WRAP;
    AddressV  = WRAP;
};

sampler alpha3 = sampler_state
{
	Texture = <a3>;
	MinFilter = LINEAR;
	MagFilter = LINEAR;
	MipFilter = POINT;
	AddressU  = WRAP;
    AddressV  = WRAP;
};

sampler texture1 = sampler_state
{
	Texture = <t1>;
	MinFilter = LINEAR;
	MagFilter = LINEAR;
	MipFilter = LINEAR;

};

sampler texture2 = sampler_state
{
	Texture = <t2>;
	MinFilter = LINEAR;
	MagFilter = LINEAR;
	MipFilter = LINEAR;
};

sampler texture3 = sampler_state
{
	Texture = <t3>;
	MinFilter = LINEAR;
	MagFilter = LINEAR;
	MipFilter = LINEAR;

};

sampler texture4 = sampler_state
{
	Texture = <t4>;
	MinFilter = LINEAR;
	MagFilter = LINEAR;
	MipFilter = LINEAR;

};

sampler texture5 = sampler_state
{
	Texture = <t5>;
	MinFilter = LINEAR;
	MagFilter = LINEAR;
	MipFilter = LINEAR;

};

sampler texture6 = sampler_state
{
	Texture = <t6>;
	MinFilter = LINEAR;
	MagFilter = LINEAR;
	MipFilter = LINEAR;
};

sampler texture7 = sampler_state
{
	Texture = <t7>;
	MinFilter = LINEAR;
	MagFilter = LINEAR;
	MipFilter = LINEAR;
};

sampler texture8 = sampler_state
{
	Texture = <t8>;
	MinFilter = LINEAR;
	MagFilter = LINEAR;
	MipFilter = LINEAR;
};

sampler texture9 = sampler_state
{
	Texture = <t9>;
	MinFilter = LINEAR;
	MagFilter = LINEAR;
	MipFilter = LINEAR;
};


sampler light = sampler_state
{
	Texture = <l>;
	MinFilter = LINEAR;
	MagFilter = LINEAR;
	MipFilter = LINEAR;
};

float4 MainPS(float2 alphaUV : TEXCOORD0, float2 colorUV : TEXCOORD1, float shade : TEXCOORD2 ) : COLOR
{
    // Sample the textures
    float3 a = tex2D(alpha, alphaUV).rgb;
    float3 a2= tex2D(alpha2, alphaUV).rgb;
    float3 a3= tex2D(alpha3, alphaUV).rgb;
    float4 c1 = tex2D(texture1, colorUV);
    float4 c2 = tex2D(texture2, colorUV);
    float4 c3 = tex2D(texture3, colorUV);
    float4 c4 = tex2D(texture4, colorUV);
    float4 c5 = tex2D(texture5, colorUV);
    float4 c6 = tex2D(texture6, colorUV);
    float4 c7 = tex2D(texture7, colorUV);
    float4 c8 = tex2D(texture8, colorUV);
    float4 c9 = tex2D(texture9, colorUV);
    float4 l  = tex2D(light, alphaUV);
    
    // Use 1st blend map
    float4 texColor = lerp(c1,c2,a.g);
	texColor = lerp(texColor,c3,a.b);
	
	// Now use 2nd blend map
	texColor = lerp(texColor,c4,a2.r);
	texColor = lerp(texColor,c5,a2.g);
	texColor = lerp(texColor,c6,a2.b);
	
	// 3rd blend map	
	texColor = lerp(texColor,c7,a3.r);
	texColor = lerp(texColor,c8,a3.g);
	texColor = lerp(texColor,c9,a3.b);
    
    return texColor * shade * l;     
}

technique TerrainTech
{
    pass P0
    {
        // Specify the vertex and pixel shader associated with this pass.
        vertexShader = compile vs_2_0 MainVS();
        pixelShader  = compile ps_2_0 MainPS();      
    }    
}
