#ifndef TERRAIN_H
#define TERRAIN_H

#include <vector>

#include "Ray.h"
#include "Utilities.h"	

class Point;
class Terrain;
class HeightMap;
class Camera;

class TerrainPatch
{
public:

	TerrainPatch();
	~TerrainPatch();
	void Release();
	
	// Creates the patch
	HRESULT CreateMesh( Terrain *pTerrain, RECT source, int index );
	
	// Updates the patch based on the new height map levels
	HRESULT Update( Terrain *pTerrain );					
	void Render(int texture);	

	LPDIRECT3DDEVICE9 mDevice;
	LPD3DXMESH	mMesh;
	RECT mMapRect;
	BoundingBox mBoundingBox;
};

/**
 * Terrain class. Handles the actual terrain generation
 */

class Terrain
{
public:
	Terrain(void);
	~Terrain(void);

	void Reset( LPDIRECT3DDEVICE9 device, HeightMap *heightmap, D3DXVECTOR3 &dirToLight, int numPatches );	
	void Release();

	void Render( Camera *cam );
	void RenderPatches( Camera *cam );	
	
	void CreatePatches(int numPatches);
	void CalculateBlendMap();	
	void CalculateLightMap();		

	DWORD GetTotalVertices() { return mTotalVertices; }
	DWORD GetTotalFaces()	 { return mTotalFaces; }
	DWORD GetTotalPatches()	 { return mTotalPatches; }		

	LPDIRECT3DTEXTURE9 GetBlendMap( size_t index ) { return index > mBlendMaps.size() ? NULL : mBlendMaps[index]; }
	LPDIRECT3DTEXTURE9 GetCalculatedBlendMap()	{ return mCalculatedBlendMap; }
	LPDIRECT3DTEXTURE9 GetLightMap()			{ return mLightMap; }
	
	std::vector<TerrainPatch*>		mPatches;		

	void SetHeightMap( HeightMap *map ) { mHeightMap = map; }

private:
	LPDIRECT3DDEVICE9	mDevice;			
	LPD3DXEFFECT		mEffect;
	HeightMap			*mHeightMap;

	std::vector<LPDIRECT3DTEXTURE9> mDiffuseMaps; // Main terrain textures
	std::vector<LPDIRECT3DTEXTURE9> mBlendMaps;	
	
	LPDIRECT3DTEXTURE9				mLightMap;	
	LPDIRECT3DTEXTURE9				mCalculatedBlendMap; // Procedural generated blendmap

	DWORD                   mTotalVertices;	// Vertex count
	DWORD					mTotalFaces;	// Primitive count
	DWORD					mTotalPatches;

	D3DXVECTOR3				mDirToLight;

	friend class TerrainPatch;

};

#endif
