#include "DXUT.h"
#include "resource.h"
#include "App.h"

#include "Terrain.h"
#include "HeightMap.h"

// Terrain size must not be greater than height map
#define TERRAINSIZE_X 256 
#define TERRAINSIZE_Y 256 

enum { IDC_SHOWPATCHES = IDC_CLIENTCONTROLID };

class TerrainSplatting : public App
{
protected:	
	virtual void RenderClient( double time, float elapsedTime )					
	{
		sprintf_s( mDebugString, "Press SPACE BAR to randomize terrain." );
		AddDebugStr( mDebugString );	

		if( mTerrain )
		{
			if(mAppDlg.GetCheckBox( IDC_SHOWPATCHES )->GetChecked() )
			{
				mTerrain->RenderPatches( &mCamera );
			}
			else
			{
				mTerrain->Render( &mCamera );
			}

			AddTriangles( mTerrain->GetTotalFaces() );
			AddVertices( mTerrain->GetTotalVertices() );
			sprintf_s( mDebugString, "Total patch visible: %d", mTerrain->GetTotalPatches() );
			AddDebugStr( mDebugString );	
		}

		// Show terrain maps for debug
		if( mTerrain && mShowGui )
		{			
			D3DXMATRIX matScale;
			float scale = 0.35f;		

			D3DSURFACE_DESC d;			
			mUserHeightMapDebugTexture->GetLevelDesc( 0, &d );						

			// Push to the bottom
			float posy = (mScreenHeight - (d.Height * scale)) / scale;			
			D3DXMatrixScaling( &matScale, scale, scale, 0.0f );					
			mSprite->Begin(0);				
			mSprite->SetTransform( &matScale );

			mSprite->Draw( mUserHeightMapDebugTexture, NULL, NULL, &D3DXVECTOR3(0.f, posy, 0.f), D3DCOLOR_ARGB(127, 255, 255, 255) );			
			mSprite->Draw( mNoiseHeightMapDebugTexture, NULL, NULL, &D3DXVECTOR3((float)d.Width, posy, 0.f), D3DCOLOR_ARGB(127, 255, 255, 255) );				
			mSprite->Draw( mFinalHeightMapDebugTexture, NULL, NULL, &D3DXVECTOR3((float)d.Width * 2, posy, 0.f), D3DCOLOR_ARGB(127, 255, 255, 255) );						
			mSprite->Draw( mTerrain->GetBlendMap( 0 ), NULL, NULL, &D3DXVECTOR3((float)d.Width * 3, posy, 0.f), D3DCOLOR_ARGB(127, 255, 255, 255) );			
			mSprite->Draw( mTerrain->GetBlendMap( 1 ), NULL, NULL, &D3DXVECTOR3((float)d.Width * 4, posy, 0.f), D3DCOLOR_ARGB(127, 255, 255, 255) );			
			mSprite->Draw( mTerrain->GetBlendMap( 2 ), NULL, NULL, &D3DXVECTOR3((float)d.Width * 5, posy, 0.f), D3DCOLOR_ARGB(127, 255, 255, 255) );			
			mSprite->Draw( mTerrain->GetLightMap() , NULL, NULL, &D3DXVECTOR3((float)d.Width * 6, posy, 0.f), D3DCOLOR_ARGB(127, 255, 255, 255) );
			mSprite->Draw( mTerrain->GetCalculatedBlendMap(), NULL, NULL, &D3DXVECTOR3((float)d.Width * 7, posy, 0.f), D3DCOLOR_ARGB(127, 255, 255, 255) );			

			mSprite->End();	

			mSprite->SetTransform( D3DXMatrixScaling( &matScale, 1.f, 1.f, 0.0f ) );

			// Draw labels
			CDXUTTextHelper txtHelper( mFont, mSprite, 15 );			
			txtHelper.SetForegroundColor(  Color::White );				

			int margin = 5;
			posy = (posy - 40.f) * scale;
			d.Width = d.Width * scale;			

			txtHelper.Begin();

			txtHelper.SetInsertionPos( margin, (int)posy );					
			txtHelper.DrawTextLine( L"Height map" );			
			txtHelper.SetInsertionPos( d.Width + margin, (int)posy );
			txtHelper.DrawTextLine( L"Random Noise" );			
			txtHelper.SetInsertionPos( d.Width * 2 + margin, (int)posy );
			txtHelper.DrawTextLine( L"Combined Final" );			
			txtHelper.SetInsertionPos( d.Width * 3 + margin, (int)posy );
			txtHelper.DrawTextLine( L"Blend map 1" );			
			txtHelper.SetInsertionPos( d.Width * 4 + margin, (int)posy );
			txtHelper.DrawTextLine( L"Blend map 2" );			
			txtHelper.SetInsertionPos( d.Width * 5 + margin, (int)posy );
			txtHelper.DrawTextLine( L"Blend map 3" );			
			txtHelper.SetInsertionPos( d.Width * 6 + margin, (int)posy );				
			txtHelper.DrawTextLine( L"Light map" );			
			txtHelper.SetInsertionPos( d.Width * 7 + margin, (int)posy );			
			txtHelper.DrawTextLine( L"Calculated blend map" );						

			txtHelper.End();
		}
	}

	virtual void LostClient()															
	{ 
		SAFE_RELEASE( mNoiseHeightMapDebugTexture );
		SAFE_RELEASE( mUserHeightMapDebugTexture );
		SAFE_RELEASE( mFinalHeightMapDebugTexture );
		SAFE_DELETE( mHeightMap[0] );
		SAFE_DELETE( mHeightMap[1] );
		SAFE_DELETE( mTerrain );
	}

	virtual void DestroyClient()														
	{ 
		SAFE_RELEASE( mNoiseHeightMapDebugTexture );
		SAFE_RELEASE( mUserHeightMapDebugTexture );
		SAFE_RELEASE( mFinalHeightMapDebugTexture );
		SAFE_DELETE( mHeightMap[0] );
		SAFE_DELETE( mHeightMap[1] );
		SAFE_DELETE( mTerrain );
	}

	virtual HRESULT OneTimeClientInit()			
	{
		mAppDlg.AddCheckBox( IDC_SHOWPATCHES, L"Show patches", 25, mCurrentGuiHeight += 24, 125, 22, false );	

		return S_OK;
	}

	void KeyboardProcClient( UINT nChar, bool bKeyDown, bool bAltDown )			
	{
		if( bKeyDown )
		{
			switch( nChar )
			{
			case VK_SPACE:
				{
					LostClient();
					ResetClient();
				}	
				break;
			}
		}
	}

	virtual HRESULT ResetClient()														
	{ 
		Progress( "Constructing terrain... ", 0.f );
		mTerrainSize.x = TERRAINSIZE_X;
		mTerrainSize.y = TERRAINSIZE_Y;
		float terrainMaxHeight = 30.f;

		Progress( "Constructing terrain... ", 0.2f );
		// Load user defined heightmap
		mHeightMap[0] = new HeightMap( mTerrainSize, terrainMaxHeight );		
		mHeightMap[0]->LoadFromFile( mDevice, "media/hm.bmp" );		
		mHeightMap[0]->GenerateHeightMapTexture( mDevice, &mUserHeightMapDebugTexture );

		int octaves = 7;
		float noiseSize = 1.5f;
		float persistence = 0.6F;

		Progress( "Constructing terrain... ", 0.5f );		
		// Create a random height map using perlin noise
		mHeightMap[1] = new HeightMap( mTerrainSize, terrainMaxHeight );		
		mHeightMap[1]->CreateRandomHeightMap( rand() % 2000, noiseSize, persistence, octaves );		
		mHeightMap[1]->GenerateHeightMapTexture( mDevice, &mNoiseHeightMapDebugTexture );		

		// Multiply user defined heightmap with the procedural generated height map
		*mHeightMap[0] *= *mHeightMap[1];	

		Progress( "Constructing terrain... ", 0.7f );		

		mTerrain = new Terrain();
		mTerrain->Reset( mDevice, mHeightMap[0], (D3DXVECTOR3)mDirLight.Direction, 10 );		

		mHeightMap[0]->GenerateHeightMapTexture( mDevice, &mFinalHeightMapDebugTexture );

		Progress( "Constructing terrain... ", 0.9f );

		return S_OK; 
	}

public:
	TerrainSplatting()
	{
		mFinalHeightMapDebugTexture	= NULL;
		mUserHeightMapDebugTexture	= NULL;
		mNoiseHeightMapDebugTexture	= NULL;
		mHeightMap[0]		= NULL;
		mHeightMap[1]		= NULL;
		mTerrain			= NULL;		
	}

	~TerrainSplatting()
	{
		SAFE_DELETE( mHeightMap[0] );
		SAFE_DELETE( mHeightMap[1] );
		SAFE_RELEASE( mFinalHeightMapDebugTexture );
		SAFE_RELEASE( mUserHeightMapDebugTexture );
		SAFE_RELEASE( mNoiseHeightMapDebugTexture );
		SAFE_DELETE( mTerrain );				
	}

private:
	HeightMap								*mHeightMap[2];
	LPDIRECT3DTEXTURE9						mFinalHeightMapDebugTexture; // Use to draw the height map generated
	LPDIRECT3DTEXTURE9						mUserHeightMapDebugTexture; 
	LPDIRECT3DTEXTURE9						mNoiseHeightMapDebugTexture; 
	Point									mTerrainSize;
	Terrain									*mTerrain;
};

//--------------------------------------------------------------------------------------
// Entry point to the program. Initializes everything and goes into a message processing 
// loop. Idle time is used to render the scene.
//--------------------------------------------------------------------------------------
int WINAPI wWinMain( HINSTANCE hInstance, HINSTANCE hPrevInstance, LPWSTR lpCmdLine, int nCmdShow )
{
	// Enable run-time memory check for debug builds.
#if defined(DEBUG) | defined(_DEBUG)
	_CrtSetDbgFlag( _CRTDBG_ALLOC_MEM_DF | _CRTDBG_LEAK_CHECK_DF );
#endif

	LOG( "Starting application..." );	
	LogOSAndAdapter();

	TerrainSplatting *pApp = new TerrainSplatting();				
	return Initialize( 800, 600, L"Terrain Splatting Demo", pApp );	
}