#include "DXUT.h"
#include "Utilities.h"

#include <vector>
#include <fstream>
#include "vertex.h"

#define COLLISION_RADIUS 0.0f
LPD3DXFONT gProgressFont = NULL;

/****************************************************************************

RandomNumber: returns a random number between iMin and iMax.

****************************************************************************/
int RandomNumber(int iMin, int iMax)
{
	if (iMin == iMax) return(iMin);
	return((rand() % (abs(iMax-iMin)+1))+iMin);
}

float RandomNumber(float fMin, float fMax)
{
	if (fMin == fMax) return(fMin);
	float fRandom = (float)rand() / (float)RAND_MAX;
	return((fRandom * (float)fabs(fMax-fMin))+fMin);
}

inline D3DXVECTOR3 RandomNumber(D3DXVECTOR3 vMin, D3DXVECTOR3 vMax)
{
	float x = RandomNumber(vMin.x, vMax.x);
	float y = RandomNumber(vMin.y, vMax.y);
	float z = RandomNumber(vMin.z, vMax.z);
	return(D3DXVECTOR3(x,y,z));
}

inline D3DXCOLOR RandomNumber(D3DXCOLOR Min, D3DXCOLOR Max)
{
	float r = RandomNumber(Min.r, Max.r);
	float g = RandomNumber(Min.g, Max.g);
	float b = RandomNumber(Min.b, Max.b);
	float a = RandomNumber(Min.a, Max.a);
	return(D3DXCOLOR(r,g,b,a));
}

char * ConvertString( char *out, WCHAR *in )
{
	size_t convertedChars = 0;
	size_t origsize = wcslen(in) + 1;

	wcstombs_s(&convertedChars, out, origsize, in, _TRUNCATE);
	return out;
}

WCHAR * ConvertString( WCHAR *out, char *in )
{
	size_t convertedChars = 0;
	size_t origsize = strlen(in) + 1;

	mbstowcs_s(&convertedChars, out, origsize, in, _TRUNCATE);
	return out;
}

WCHAR * ConvertString( char *in )
{
	static WCHAR out[512];

	size_t convertedChars = 0;
	size_t origsize = strlen(in) + 1;

	mbstowcs_s(&convertedChars, out, origsize, in, _TRUNCATE);
	return out;
}

bool InitGlobalVariables()
{
	LPDIRECT3DDEVICE9 device = DXUTGetD3D9Device();	

	if( FAILED( D3DXCreateFont( device, 40, 0, FW_BOLD, 1, FALSE, DEFAULT_CHARSET,
			OUT_DEFAULT_PRECIS, DEFAULT_QUALITY, DEFAULT_PITCH | FF_DONTCARE, L"Arial", &gProgressFont ) ) )
		{
			LOG_ERROR ( "Failed to create font for progress indicator" );
			return false;
		}
	
	return true;
}

void DestroyGlobalVariables()
{
	SAFE_RELEASE( gProgressFont );
}

void Progress( std::string text, float prc )
{
	LPDIRECT3DDEVICE9 device = DXUTGetD3D9Device();	
	const D3DSURFACE_DESC* pBackBufferSurfaceDesc = DXUTGetD3D9BackBufferSurfaceDesc();	
	long width = pBackBufferSurfaceDesc->Width;
	long height = pBackBufferSurfaceDesc->Height;	
	
	int rectwidth = 400;
	int rectheight = 50;

	int left = ( width / 2) - (rectwidth / 2);
	int top = ( height / 2) - (rectheight / 2);	

	device->Clear(0, 0, D3DCLEAR_TARGET | D3DCLEAR_ZBUFFER, 0xffffffff, 1.0f, 0L);
	device->BeginScene();
	
	RECT rc = {left, top - 100, left + rectwidth, top + rectheight};
	gProgressFont->DrawText(NULL, ConvertString((char*)text.c_str()), -1, &rc, DT_CENTER | DT_VCENTER | DT_NOCLIP, 0xff000000);
	
	//Progress bar
	D3DRECT r;
	r.x1 = left; r.x2 = left + rectwidth;
	r.y1 = top; r.y2 = top + rectheight;
	device->Clear(1, &r, D3DCLEAR_TARGET | D3DCLEAR_ZBUFFER, 0xff000000, 1.0f, 0L);
	r.x1 = left + 2; r.x2 = left + rectwidth - 2;
	r.y1 = top + 2; r.y2 = top + rectheight - 2;
	device->Clear(1, &r, D3DCLEAR_TARGET | D3DCLEAR_ZBUFFER, 0xffffffff, 1.0f, 0L);
	r.x1 = left + 2; r.x2 = r.x1 + (rectwidth - 4) * prc;
	r.y1 = top + 2; r.y2 = top + rectheight - 2;
	device->Clear(1, &r, D3DCLEAR_TARGET | D3DCLEAR_ZBUFFER, D3DCOLOR_RGBA( 135, 206, 250, 255 ), 1.0f, 0L);

	device->EndScene();
	device->Present( 0, 0, 0, 0 );
}

char * Commify(double f)
{
	static char s[1000];
	char *p;
	sprintf_s(s,"%f",f);
	p = s+strlen(s);
	while( *--p=='0' ){ *p='\0'; };
	if( *p=='.' ){ *p='\0'; }
	p = strrchr(s,'.');
	if( p == 0 ){ p = s+strlen(s); }
	while( (p-=3) > s ){
		memmove(p+1,p,strlen(p));
		*p=',';
	}
	return s;
}

void LogOSAndAdapter()
{
	LOG( "Dumping processor specs" );
	LOG( "-----------------------" );
	// Get and display the name of the computer. 
	char	convertBuf[512];
	TCHAR	infoBuf[32767];
	DWORD	bufCharCount = 32767;
	GetComputerName( infoBuf, &bufCharCount );	
	LOG( "Computer name: " << ConvertString( convertBuf, infoBuf ) );

	// Get Windows version
	OSVERSIONINFO osver;
	ZeroMemory( &osver, sizeof(osver) );
	osver.dwOSVersionInfoSize = sizeof(osver);	
	GetVersionEx( &osver );
	const char* os = "";
	const int version = osver.dwMajorVersion * 100 + osver.dwMinorVersion;
	switch ( osver.dwPlatformId ) {

	case VER_PLATFORM_WIN32s:
		os = "Win31 with Win32s";
		break;

	case VER_PLATFORM_WIN32_WINDOWS:
		switch ( version ) {
	case 400: os = "Win95"; break;
	case 410: os = "Win98"; break;
	case 490: os = "WinME"; break;
	default: os = "Win?? (95-class)"; break;
		}
		break;

	case VER_PLATFORM_WIN32_NT:
		switch ( version ) {
	case 351: os = "WinNT 3.51"; break;
	case 400: os = "WinNT 4.0"; break;
	case 500: os = "Win2K"; break;
	case 501: os = "WinXP"; break;
	case 502: os = "Windows Server 2003"; break;
	case 600: os = "Windows Vista"; break;
	default: os = "Win?? (NT-class)"; break;
		}
		break;
	}

	// Get DirectX version if available
	TCHAR dx[164];
	HKEY key;	
	if ( RegOpenKeyEx( HKEY_LOCAL_MACHINE, L"SOFTWARE\\Microsoft\\DirectX", 0, KEY_READ, &key ) == ERROR_SUCCESS ) 
	{
		DWORD size = sizeof(dx)-1;
		RegQueryValueEx( key, L"Version", 0, 0, (LPBYTE)dx, &size );
		dx[ size ] = 0;
		RegCloseKey( key );
	}

	// now dump os and directx version
	char buf[256];
	sprintf_s( buf, "%s, Version %d.%d.%d %s, DirectX Version %s",
		os, osver.dwMajorVersion, osver.dwMinorVersion, osver.dwBuildNumber, 
		((osver.szCSDVersion!=NULL) ? osver.szCSDVersion : L""), ConvertString( convertBuf, dx ) );
	LOG( "OS: " << buf );    

	// dump cpu information
	if( RegOpenKeyEx(HKEY_LOCAL_MACHINE, L"HARDWARE\\DESCRIPTION\\System\\CentralProcessor\\0",
		0,KEY_READ,&key) == ERROR_SUCCESS )
	{
		DWORD dwType;		
		unsigned __int64	dwSpeed;
		DWORD dwSize = sizeof(dwSpeed);
		RegQueryValueEx(key, L"~MHz", NULL, &dwType, (LPBYTE) &dwSpeed, &dwSize);	

		LPTSTR				sIdentifier;	// eg: x86 Family 6 Model 6 Stepping 2
		LPTSTR				sProcessorName;	// eg: AMD Athlon(tm) XP 1600+
		LPTSTR				sVendorName;	// eg: AuthenticAMD

		RegQueryValueEx(key,	L"Identifier", NULL, &dwType, NULL, &dwSize);	
		sIdentifier = new TCHAR[dwSize];	
		RegQueryValueEx(key,	L"Identifier", NULL, &dwType, (LPBYTE)sIdentifier, &dwSize);

		RegQueryValueEx(key,	L"ProcessorNameString", NULL, &dwType, NULL, &dwSize);	
		sProcessorName = new TCHAR[dwSize];	
		RegQueryValueEx(key,	L"ProcessorNameString", NULL, &dwType, (LPBYTE)sProcessorName, &dwSize);

		RegQueryValueEx(key,	L"VendorIdentifier", NULL, &dwType, NULL, &dwSize);	
		sVendorName = new TCHAR[dwSize];	
		RegQueryValueEx(key,	L"VendorIdentifier", NULL, &dwType, (LPBYTE)sVendorName, &dwSize);

		RegCloseKey( key );

		char temp[512];				
		LOG( "Identifier: " << ConvertString( temp, sIdentifier) );
		LOG( "ProcessorNameString: " << ConvertString( temp, sProcessorName) );
		LOG( "VendorIdentifier: " << ConvertString( temp, sVendorName) );
		LOG( "~MHz: " << Commify( (double) dwSpeed ) );

		delete sIdentifier;
		delete sProcessorName;
		delete sVendorName;
	}

	// Dump memory
	MEMORYSTATUSEX statex;
	ZeroMemory( &statex, sizeof( statex ) );
	statex.dwLength = sizeof (statex);

	GlobalMemoryStatusEx (&statex);
	LOG( "Total memory: " << Commify( statex.ullTotalPhys / 1024 / 1024 ) << "(MB)" );	

	/*	
	// LOG( "Percent of memory in use: " << Commify( statex.dwMemoryLoad ) );
	DumpWrite( file, "Total MB of paging file", Commify( statex.ullTotalPageFile / 1024 / 1024 ) );
	DumpWrite( file, "Total MB of virtual memory", Commify( statex.ullTotalVirtual / 1024 / 1024 ) );
	*/

	LOG( "Free memory: " << Commify( statex.ullAvailPhys / 1024 / 1024 ) << "(MB)" );
	/*
	DumpWrite( file, "Free MB of paging file", Commify( statex.ullAvailPageFile / 1024 / 1024) );	
	DumpWrite( file, "Free MB of virtual memory", Commify( statex.ullAvailVirtual / 1024 / 1024 ) );
	DumpWrite( file, "Free MB of extended memory", Commify( statex.ullAvailExtendedVirtual / 1024 / 1024 ) );
	*/
	
	LOG( "------------------------" );
	LOG( "Dumping video card specs" );	
	LOG( "------------------------" );
	IDirect3D9* pD3D9 = NULL;
	pD3D9 = Direct3DCreate9( D3D_SDK_VERSION );
	UINT dwAdapterCount = pD3D9->GetAdapterCount();
	for( UINT iAdapter = 0; iAdapter < dwAdapterCount; iAdapter++ )
	{
		D3DADAPTER_IDENTIFIER9 id;
		ZeroMemory( &id, sizeof( D3DADAPTER_IDENTIFIER9 ) );
		pD3D9->GetAdapterIdentifier( iAdapter, 0, &id );		
		char ver_s[ 64 ];
		const LARGE_INTEGER& ver = id.DriverVersion;
		sprintf_s( ver_s, "%i.%i.%i.%i", HIWORD(ver.HighPart), LOWORD(ver.HighPart), HIWORD(ver.LowPart), LOWORD(ver.LowPart) );			

		LOG( id.Description << "(adapter " << iAdapter << ")" );
		LOG( "Driver: " << id.Driver << " " << ver_s );			

		HMONITOR hMonitor = pD3D9->GetAdapterMonitor( iAdapter );
		// LOG( "hMonitor: 0x%0.8x" << ( DWORD_PTR )hMonitor );			

		MONITORINFOEX mi;
		mi.cbSize = sizeof( MONITORINFOEX );
		GetMonitorInfo( hMonitor, &mi );
		LOG( "hMonitor Device Name: " <<  mi.szDevice );

		DWORD dwAvailableVidMem, dwAdapterRAM;
		UINT dwAvailableTextureMem;
		if( SUCCEEDED( GetVideoMemoryViaDirectDraw( hMonitor, &dwAvailableVidMem ) ) )
		{
			LOG( "AvailableVidMem: " << dwAvailableVidMem / 1024 / 1024 << "(MB)" );                
		}
		GetVideoMemoryViaD3D9( hMonitor, &dwAvailableTextureMem );
		LOG( "AvailableTexMem: " << dwAvailableTextureMem / 1024 / 1024 << "(MB)" );                
		GetVideoMemoryViaWMI( hMonitor, &dwAdapterRAM );
		LOG( "AdapterRam: " << dwAdapterRAM / 1024 / 1024 << "(MB)" );    
		if( iAdapter != dwAdapterCount - 1 )
		{
			LOG( "------------------------" );
		}		
	}
	SAFE_RELEASE( pD3D9 );
	LOG( "----------Done dumping specs------------" );
}

#include <ddraw.h>

typedef HRESULT ( WINAPI* LPDIRECTDRAWCREATE )( GUID FAR *lpGUID, LPDIRECTDRAW FAR *lplpDD, IUnknown FAR *pUnkOuter );

struct DDRAW_MATCH
{
	GUID guid;
	HMONITOR hMonitor;
	CHAR strDriverName[512];
	bool bFound;
};


//-----------------------------------------------------------------------------
BOOL WINAPI DDEnumCallbackEx( GUID FAR* lpGUID, LPSTR lpDriverDescription, LPSTR lpDriverName, LPVOID lpContext, HMONITOR hm )
{
	DDRAW_MATCH* pDDMatch = ( DDRAW_MATCH* ) lpContext;
	if( pDDMatch->hMonitor == hm )
	{
		pDDMatch->bFound = true;
		strcpy_s( pDDMatch->strDriverName, 512, lpDriverName );
		memcpy( &pDDMatch->guid, lpGUID, sizeof( GUID ) );
	}
	return TRUE;
}


//-----------------------------------------------------------------------------
HRESULT GetVideoMemoryViaDirectDraw( HMONITOR hMonitor, DWORD* pdwAvailableVidMem )
{
	LPDIRECTDRAW pDDraw = NULL;
	LPDIRECTDRAWENUMERATEEXA pDirectDrawEnumerateEx = NULL;
	HRESULT hr;
	bool bGotMemory = false;
	*pdwAvailableVidMem = 0;

	HINSTANCE hInstDDraw;
	LPDIRECTDRAWCREATE pDDCreate = NULL;

	hInstDDraw = LoadLibrary( L"ddraw.dll" );
	if( hInstDDraw )
	{
		DDRAW_MATCH match;
		ZeroMemory( &match, sizeof( DDRAW_MATCH ) );
		match.hMonitor = hMonitor;

		pDirectDrawEnumerateEx = ( LPDIRECTDRAWENUMERATEEXA )GetProcAddress( hInstDDraw, "DirectDrawEnumerateExA" );

		if( pDirectDrawEnumerateEx )
		{
			hr = pDirectDrawEnumerateEx( DDEnumCallbackEx, ( VOID* )&match, DDENUM_ATTACHEDSECONDARYDEVICES );
		}

		pDDCreate = ( LPDIRECTDRAWCREATE )GetProcAddress( hInstDDraw, "DirectDrawCreate" );
		if( pDDCreate )
		{
			pDDCreate( &match.guid, &pDDraw, NULL );

			LPDIRECTDRAW7 pDDraw7;
			if( SUCCEEDED( pDDraw->QueryInterface( IID_IDirectDraw7, ( VOID** )&pDDraw7 ) ) )
			{
				DDSCAPS2 ddscaps;
				ZeroMemory( &ddscaps, sizeof( DDSCAPS2 ) );
				ddscaps.dwCaps = DDSCAPS_VIDEOMEMORY | DDSCAPS_LOCALVIDMEM;
				hr = pDDraw7->GetAvailableVidMem( &ddscaps, pdwAvailableVidMem, NULL );
				if( SUCCEEDED( hr ) )
					bGotMemory = true;
				pDDraw7->Release();
			}
		}
		FreeLibrary( hInstDDraw );
	}


	if( bGotMemory )
		return S_OK;
	else
		return E_FAIL;
}

HRESULT GetVideoMemoryViaD3D9( HMONITOR hMonitor, UINT* pdwAvailableTextureMem )
{
    HRESULT hr;
    bool bGotMemory = false;
    *pdwAvailableTextureMem = 0;

    IDirect3D9* pD3D9 = NULL;
    pD3D9 = Direct3DCreate9( D3D_SDK_VERSION );
    if( pD3D9 )
    {
        UINT dwAdapterCount = pD3D9->GetAdapterCount();
        for( UINT iAdapter = 0; iAdapter < dwAdapterCount; iAdapter++ )
        {
            IDirect3DDevice9* pd3dDevice = NULL;

            HMONITOR hAdapterMonitor = pD3D9->GetAdapterMonitor( iAdapter );
            if( hMonitor != hAdapterMonitor )
                continue;

            HWND hWnd = GetDesktopWindow();

            D3DPRESENT_PARAMETERS pp;
            ZeroMemory( &pp, sizeof( D3DPRESENT_PARAMETERS ) );
            pp.BackBufferWidth = 800;
            pp.BackBufferHeight = 600;
            pp.BackBufferFormat = D3DFMT_R5G6B5;
            pp.BackBufferCount = 1;
            pp.MultiSampleType = D3DMULTISAMPLE_NONE;
            pp.MultiSampleQuality = 0;
            pp.SwapEffect = D3DSWAPEFFECT_DISCARD;
            pp.hDeviceWindow = hWnd;
            pp.Windowed = TRUE;

            pp.EnableAutoDepthStencil = FALSE;
            pp.Flags = 0;
            pp.FullScreen_RefreshRateInHz = 0;
            pp.PresentationInterval = D3DPRESENT_INTERVAL_DEFAULT;

            hr = pD3D9->CreateDevice( iAdapter, D3DDEVTYPE_HAL, hWnd,
                                      D3DCREATE_SOFTWARE_VERTEXPROCESSING, &pp, &pd3dDevice );
            if( SUCCEEDED( hr ) )
            {
                *pdwAvailableTextureMem = pd3dDevice->GetAvailableTextureMem();
                bGotMemory = true;
                SAFE_RELEASE( pd3dDevice );
            }
        }

        SAFE_RELEASE( pD3D9 );
    }

    if( bGotMemory )
        return S_OK;
    else
        return E_FAIL;
}

HRESULT GetDeviceIDFromHMonitor( HMONITOR hm, WCHAR* strDeviceID, int cchDeviceID )
{

    HINSTANCE hInstDDraw;

    hInstDDraw = LoadLibrary( L"ddraw.dll" );
    if( hInstDDraw )
    {
        DDRAW_MATCH match;
        ZeroMemory( &match, sizeof( DDRAW_MATCH ) );
        match.hMonitor = hm;

        LPDIRECTDRAWENUMERATEEXA pDirectDrawEnumerateEx = NULL;
        pDirectDrawEnumerateEx = ( LPDIRECTDRAWENUMERATEEXA )GetProcAddress( hInstDDraw, "DirectDrawEnumerateExA" );

        if( pDirectDrawEnumerateEx )
            pDirectDrawEnumerateEx( DDEnumCallbackEx, ( VOID* )&match, DDENUM_ATTACHEDSECONDARYDEVICES );

        if( match.bFound )
        {
            LONG iDevice = 0;
            DISPLAY_DEVICEA dispdev;

            ZeroMemory( &dispdev, sizeof( dispdev ) );
            dispdev.cb = sizeof( dispdev );

            while( EnumDisplayDevicesA( NULL, iDevice, ( DISPLAY_DEVICEA* )&dispdev, 0 ) )
            {
                // Skip devices that are monitors that echo another display
                if( dispdev.StateFlags & DISPLAY_DEVICE_MIRRORING_DRIVER )
                {
                    iDevice++;
                    continue;
                }

                // Skip devices that aren't attached since they cause problems
                if( ( dispdev.StateFlags & DISPLAY_DEVICE_ATTACHED_TO_DESKTOP ) == 0 )
                {
                    iDevice++;
                    continue;
                }

                if( _stricmp( match.strDriverName, dispdev.DeviceName ) == 0 )
                {
                    MultiByteToWideChar( CP_ACP, 0, dispdev.DeviceID, -1, strDeviceID, cchDeviceID );
                    return S_OK;
                }

                iDevice++;
            }
        }

        FreeLibrary( hInstDDraw );
    }

    return E_FAIL;
}

#include <wbemidl.h>

typedef BOOL ( WINAPI* PfnCoSetProxyBlanket )( IUnknown* pProxy, DWORD dwAuthnSvc, DWORD dwAuthzSvc,
                                               OLECHAR* pServerPrincName, DWORD dwAuthnLevel, DWORD dwImpLevel,
                                               RPC_AUTH_IDENTITY_HANDLE pAuthInfo, DWORD dwCapabilities );

HRESULT GetVideoMemoryViaWMI( HMONITOR hMonitor, DWORD* pdwAdapterRam )
{
    WCHAR strInputDeviceID[512];
    GetDeviceIDFromHMonitor( hMonitor, strInputDeviceID, 512 );

    HRESULT hr;
    bool bGotMemory = false;
    HRESULT hrCoInitialize = S_OK;
    IWbemLocator* pIWbemLocator = NULL;
    IWbemServices* pIWbemServices = NULL;
    BSTR pNamespace = NULL;

    *pdwAdapterRam = 0;
    hrCoInitialize = CoInitialize( 0 );

    hr = CoCreateInstance( CLSID_WbemLocator,
                           NULL,
                           CLSCTX_INPROC_SERVER,
                           IID_IWbemLocator,
                           ( LPVOID* )&pIWbemLocator );
#ifdef PRINTF_DEBUGGING
    if( FAILED( hr ) ) wprintf( L"WMI: CoCreateInstance failed: 0x%0.8x\n", hr );
#endif

    if( SUCCEEDED( hr ) && pIWbemLocator )
    {
        // Using the locator, connect to WMI in the given namespace.
        pNamespace = SysAllocString( L"\\\\.\\root\\cimv2" );

        hr = pIWbemLocator->ConnectServer( pNamespace, NULL, NULL, 0L,
                                           0L, NULL, NULL, &pIWbemServices );
#ifdef PRINTF_DEBUGGING
        if( FAILED( hr ) ) wprintf( L"WMI: pIWbemLocator->ConnectServer failed: 0x%0.8x\n", hr );
#endif
        if( SUCCEEDED( hr ) && pIWbemServices != NULL )
        {
            HINSTANCE hinstOle32 = NULL;

            hinstOle32 = LoadLibraryW( L"ole32.dll" );
            if( hinstOle32 )
            {
                PfnCoSetProxyBlanket pfnCoSetProxyBlanket = NULL;

                pfnCoSetProxyBlanket = ( PfnCoSetProxyBlanket )GetProcAddress( hinstOle32, "CoSetProxyBlanket" );
                if( pfnCoSetProxyBlanket != NULL )
                {
                    // Switch security level to IMPERSONATE. 
                    pfnCoSetProxyBlanket( pIWbemServices, RPC_C_AUTHN_WINNT, RPC_C_AUTHZ_NONE, NULL,
                                          RPC_C_AUTHN_LEVEL_CALL, RPC_C_IMP_LEVEL_IMPERSONATE, NULL, 0 );
                }

                FreeLibrary( hinstOle32 );
            }

            IEnumWbemClassObject* pEnumVideoControllers = NULL;
            BSTR pClassName = NULL;

            pClassName = SysAllocString( L"Win32_VideoController" );

            hr = pIWbemServices->CreateInstanceEnum( pClassName, 0,
                                                     NULL, &pEnumVideoControllers );
#ifdef PRINTF_DEBUGGING
            if( FAILED( hr ) ) wprintf( L"WMI: pIWbemServices->CreateInstanceEnum failed: 0x%0.8x\n", hr );
#endif

            if( SUCCEEDED( hr ) && pEnumVideoControllers )
            {
                IWbemClassObject* pVideoControllers[10] = {0};
                DWORD uReturned = 0;
                BSTR pPropName = NULL;

                // Get the first one in the list
                pEnumVideoControllers->Reset();
                hr = pEnumVideoControllers->Next( 5000,             // timeout in 5 seconds
                                                  10,                  // return the first 10
                                                  pVideoControllers,
                                                  &uReturned );
#ifdef PRINTF_DEBUGGING
                if( FAILED( hr ) ) wprintf( L"WMI: pEnumVideoControllers->Next failed: 0x%0.8x\n", hr );
                if( uReturned == 0 ) wprintf( L"WMI: pEnumVideoControllers uReturned == 0\n" );
#endif

                VARIANT var;
                if( SUCCEEDED( hr ) )
                {
                    bool bFound = false;
                    for( UINT iController = 0; iController < uReturned; iController++ )
                    {
                        pPropName = SysAllocString( L"PNPDeviceID" );
                        hr = pVideoControllers[iController]->Get( pPropName, 0L, &var, NULL, NULL );
#ifdef PRINTF_DEBUGGING
                        if( FAILED( hr ) )
                            wprintf( L"WMI: pVideoControllers[iController]->Get PNPDeviceID failed: 0x%0.8x\n", hr );
#endif
                        if( SUCCEEDED( hr ) )
                        {
                            if( wcsstr( var.bstrVal, strInputDeviceID ) != 0 )
                                bFound = true;
                        }
                        VariantClear( &var );
                        if( pPropName ) SysFreeString( pPropName );

                        if( bFound )
                        {
                            pPropName = SysAllocString( L"AdapterRAM" );
                            hr = pVideoControllers[iController]->Get( pPropName, 0L, &var, NULL, NULL );
#ifdef PRINTF_DEBUGGING
                            if( FAILED( hr ) )
                                wprintf( L"WMI: pVideoControllers[iController]->Get AdapterRAM failed: 0x%0.8x\n",
                                         hr );
#endif
                            if( SUCCEEDED( hr ) )
                            {
                                bGotMemory = true;
                                *pdwAdapterRam = var.ulVal;
                            }
                            VariantClear( &var );
                            if( pPropName ) SysFreeString( pPropName );
                            break;
                        }
                        SAFE_RELEASE( pVideoControllers[iController] );
                    }
                }
            }

            if( pClassName )
                SysFreeString( pClassName );
            SAFE_RELEASE( pEnumVideoControllers );
        }

        if( pNamespace )
            SysFreeString( pNamespace );
        SAFE_RELEASE( pIWbemServices );
    }

    SAFE_RELEASE( pIWbemLocator );

    if( SUCCEEDED( hrCoInitialize ) )
        CoUninitialize();

    if( bGotMemory )
        return S_OK;
    else
        return E_FAIL;
}